' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.Diagnostics
Imports System.Collections.Generic
Imports System.Linq

Namespace ServiceModelEx
   Public Module ArrayExtensions
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function ConvertAll(Of T,U)(ByVal array() As T, ByVal converter As Converter(Of T,U)) As U()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.ConvertAll(converter).ToArray()
	  End Function

	  <System.Runtime.CompilerServices.Extension> _
	  Public Function SkipWhile(Of T)(ByVal array() As T, ByVal match As Predicate(Of T)) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 If match Is Nothing Then
			Throw New ArgumentNullException("match")
		 End If
            Dim func As Func(Of T, Boolean) = Function(objToMatch) match(objToMatch)
		 Return array.SkipWhile(func).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Take(Of T)(ByVal array() As T, ByVal count As Integer) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 If count < 0 Then
			Throw New IndexOutOfRangeException("count")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Take(count).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function TakeWhile(Of T)(ByVal array() As T, ByVal match As Predicate(Of T)) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 If match Is Nothing Then
			Throw New ArgumentNullException("match")
		 End If
            Dim func As Func(Of T, Boolean) = Function(objToMatch) match(objToMatch)
		 Return array.TakeWhile(func).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Skip(Of T)(ByVal array() As T, ByVal count As Integer) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 If count < 0 Then
			Throw New IndexOutOfRangeException("count")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Skip(count).ToArray()
	  End Function
	  Public Function Repeat(Of T)(ByVal element As T, ByVal count As Integer) As T()
		 If count <= 0 Then
			Throw New ArgumentOutOfRangeException("count")
		 End If
		 Return Enumerable.Repeat(Of T)(element,count).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Concat(Of T)(ByVal first() As T, ByVal second() As T) As T()
		 If first Is Nothing Then
			Throw New ArgumentNullException("first")
		 End If
		 If second Is Nothing Then
			Throw New ArgumentNullException("second")
		 End If
		 Dim enumerable As IEnumerable(Of T) = first
		 Return enumerable.Concat(second).ToArray()
	  End Function

	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Reverse(Of T)(ByVal array() As T) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Reverse().ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Union(Of T As IEquatable(Of T))(ByVal array1() As T, ByVal array2() As T) As T()
		 If array1 Is Nothing Then
			Throw New ArgumentNullException("array1")
		 End If
		 If array2 Is Nothing Then
			Throw New ArgumentNullException("array2")
		 End If

		 Dim enumerable1 As IEnumerable(Of T) = array1
		 Return enumerable1.Union(array2).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Complement(Of T As IEquatable(Of T))(ByVal array1() As T, ByVal array2() As T) As T()
		 If array1 Is Nothing Then
			Throw New ArgumentNullException("array1")
		 End If
		 If array2 Is Nothing Then
			Throw New ArgumentNullException("array2")
		 End If
		 Dim enumerable1 As IEnumerable(Of T) = array1
		 Return enumerable1.Complement(array2).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Except(Of T As IEquatable(Of T))(ByVal array1() As T, ByVal array2() As T) As T()
		 If array1 Is Nothing Then
			Throw New ArgumentNullException("array1")
		 End If
		 If array2 Is Nothing Then
			Throw New ArgumentNullException("array2")
		 End If
		 Dim enumerable1 As IEnumerable(Of T) = array1
		 Return enumerable1.Except(array2).ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Intersect(Of T As IEquatable(Of T))(ByVal array1() As T, ByVal array2() As T) As T()
		 If array1 Is Nothing Then
			Throw New ArgumentNullException("array1")
		 End If
		 If array2 Is Nothing Then
			Throw New ArgumentNullException("array2")
		 End If
		 Dim enumerable1 As IEnumerable(Of T) = array1
		 Return enumerable1.Intersect(array2).ToArray()
	  End Function

	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Distinct(Of T)(ByVal array() As T) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Distinct().ToArray()
	  End Function


	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Sort(Of T)(ByVal array() As T) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Sort().ToArray()
	  End Function
	  <System.Runtime.CompilerServices.Extension> _
	  Public Function Where(Of T)(ByVal array() As T, ByVal match As Predicate(Of T)) As T()
		 If array Is Nothing Then
			Throw New ArgumentNullException("array")
		 End If
		 If match Is Nothing Then
			Throw New ArgumentNullException("match")
		 End If
            Dim func As Func(Of T, Boolean) = Function(objToMatch) match(objToMatch)
		 Dim enumerable As IEnumerable(Of T) = array
		 Return enumerable.Where(func).ToArray()
	  End Function
   End Module
End Namespace