' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.Diagnostics
Imports System.Collections.Generic
Imports System.Linq
Imports System.Collections

Namespace ServiceModelEx
    Public Module CollectionExtensions
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub ForEach(Of T)(ByVal collection As IEnumerable(Of T), ByVal action As Action(Of T))
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If action Is Nothing Then
                Throw New ArgumentNullException("action")
            End If
            For Each item As T In collection
                action(item)
            Next item
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Function ConvertAll(Of T, U)(ByVal collection As IEnumerable(Of T), ByVal converter As Converter(Of T, U)) As IEnumerable(Of U)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            If converter Is Nothing Then
                Throw New ArgumentNullException("converter")
            End If
            Dim items As New List(Of U)
            For Each item As T In collection
                items.Add(converter(item))
            Next item
            Return items
        End Function
        <System.Runtime.CompilerServices.Extension()> _
        Public Function Complement(Of T)(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim items As New List(Of T)
            For Each item As T In collection1
                If collection2.Contains(item) = False Then
                    items.Add(item)
                End If
            Next item
            Return items
        End Function
        Public Function Except(Of T As IEquatable(Of T))(ByVal collection1 As IEnumerable(Of T), ByVal collection2 As IEnumerable(Of T)) As IEnumerable(Of T)
            Dim complement1 As IEnumerable(Of T) = Complement(collection1, collection2)
            Dim complement2 As IEnumerable(Of T) = Complement(collection2, collection1)
            Return complement1.Union(complement2)
        End Function
        <System.Runtime.CompilerServices.Extension()> _
        Public Function Sort(Of T)(ByVal collection As IEnumerable(Of T)) As IEnumerable(Of T)
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Dim list As New List(Of T)(collection)
            list.Sort()
            Dim items As New List(Of T)
            For Each item As T In list
                items.Add(item)
            Next item
            Return items
        End Function
        <System.Runtime.CompilerServices.Extension()> _
        Public Function FindIndex(Of T As IEquatable(Of T))(ByVal collection As IEnumerable(Of T), ByVal value As T) As Integer
            If collection Is Nothing Then
                Throw New ArgumentNullException("collection")
            End If
            Using iterator As IEnumerator(Of T) = collection.GetEnumerator()
                Dim index As Integer = 0

                Do While iterator.MoveNext()
                    If iterator.Current.Equals(value) = False Then
                        index += 1
                    Else
                        Return index
                    End If
                Loop
                Return -1
            End Using
        End Function
    End Module
End Namespace