﻿' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net



Imports Microsoft.VisualBasic
Imports System
Imports System.Collections.Generic
Imports System.Reflection
Imports System.Diagnostics
Imports System.ServiceModel
Imports Microsoft.ServiceBus
Imports System.Runtime.CompilerServices
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Channels
Imports System.ServiceModel.Description

Namespace ServiceModelEx.ServiceBus
    Public MustInherit Class EventsHost
        Implements IServiceBusProperties
        Private m_Credential As TransportClientEndpointBehavior

        'For service cert lookup
        Private m_ServiceCertLocation As StoreLocation
        Private m_ServiceCertStoreName As StoreName
        Private m_ServiceCertFindType As X509FindType
        Private m_ServiceCertFindValue As Object

        'For message security
        Private m_ApplicationName As String
        Private m_UseProviders As Boolean
        Private m_Anonymous As Boolean

        'Managing the host-per-event
        Private privateHosts As Dictionary(Of Type, Dictionary(Of String, ServiceBusHost))
        Protected Property Hosts() As Dictionary(Of Type, Dictionary(Of String, ServiceBusHost))
            Get
                Return privateHosts
            End Get
            Set(ByVal value As Dictionary(Of Type, Dictionary(Of String, ServiceBusHost)))
                privateHosts = value
            End Set
        End Property

        Private m_SericeType As Type
        Private privateBaseAddresses As String()
        Protected Property BaseAddresses() As String()
            Get
                Return privateBaseAddresses
            End Get
            Set(ByVal value As String())
                privateBaseAddresses = value
            End Set
        End Property

        Private m_SingletonInstance As Object

        Private privateRelayBinding As NetOnewayRelayBinding
        Protected Property RelayBinding() As NetOnewayRelayBinding
            Get
                Return privateRelayBinding
            End Get
            Set(ByVal value As NetOnewayRelayBinding)
                privateRelayBinding = value
            End Set
        End Property

        'Service bus credentials
        Private m_ServiceBusPassword As String
        Private m_ServiceBusCertLocation As StoreLocation
        Private m_ServiceBusCertStoreName As StoreName
        Private m_ServiceBusCertFindType As X509FindType
        Private m_ServiceBusCertFindValue As Object

        Public Sub New(ByVal singletonInstance As Object, ByVal baseAddress As String)
            Me.New(singletonInstance, New String() {baseAddress})
        End Sub

        Public Sub New(ByVal singletonInstance As Object, ByVal baseAddresses() As String)
            Hosts = New Dictionary(Of Type, Dictionary(Of String, ServiceBusHost))()

            Debug.Assert(baseAddresses IsNot Nothing)
            Debug.Assert(baseAddresses.Length > 0)

            m_SingletonInstance = singletonInstance

            For index As Integer = 0 To baseAddresses.Length - 1
                If baseAddresses(index).EndsWith("/") = False Then
                    baseAddresses(index) &= "/"
                End If
            Next index

            baseAddresses = baseAddresses

            'Try to guess a certificate 
            ConfigureAnonymousMessageSecurity(ServiceBusHelper.ExtractSolution(New Uri(baseAddresses(0))))
        End Sub
        Public Sub New(ByVal serviceType As Type, ByVal baseAddress As String)
            Me.New(serviceType, New String() {baseAddress})
        End Sub
        Public Sub New(ByVal serviceType As Type, ByVal baseAddresses() As String)
            Hosts = New Dictionary(Of Type, Dictionary(Of String, ServiceBusHost))()

            Debug.Assert(baseAddresses IsNot Nothing)
            Debug.Assert(baseAddresses.Length > 0)

            m_SericeType = serviceType

            For index As Integer = 0 To baseAddresses.Length - 1
                If baseAddresses(index).EndsWith("/") = False Then
                    baseAddresses(index) &= "/"
                End If
            Next index
            baseAddresses = baseAddresses

            'Try to guess a certificate 
            ConfigureAnonymousMessageSecurity(ServiceBusHelper.ExtractSolution(New Uri(baseAddresses(0))))
        End Sub


        Public Sub SetServiceBusCertificate(ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            m_ServiceBusCertFindValue = findValue
            m_ServiceBusCertLocation = location
            m_ServiceBusCertStoreName = storeName
            m_ServiceBusCertFindType = findType

            m_Credential = New TransportClientEndpointBehavior()
            m_Credential.CredentialType = TransportClientCredentialType.X509Certificate
            m_Credential.Credentials.ClientCertificate.SetCertificate(location, storeName, findType, findValue)
        End Sub
        Public Sub SetServiceBusCertificate(ByVal subjectName As String)
            SetServiceBusCertificate(subjectName, StoreLocation.LocalMachine, StoreName.My, X509FindType.FindBySubjectName)
        End Sub
        Public Sub SetServiceBusPassword(ByVal password As String)
            m_ServiceBusPassword = password

            m_Credential = New TransportClientEndpointBehavior()
            m_Credential.CredentialType = TransportClientCredentialType.UserNamePassword
            m_Credential.Credentials.UserName.UserName = ServiceBusHelper.ExtractSolution(New Uri(BaseAddresses(0)))
            m_Credential.Credentials.UserName.Password = password
        End Sub

        Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String)
            ConfigureAnonymousMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My)
        End Sub
        Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName)
            ConfigureAnonymousMessageSecurity(location, storeName, X509FindType.FindBySubjectName, serviceCert)
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub ConfigureAnonymousMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object)
            m_ServiceCertLocation = location
            m_ServiceCertStoreName = storeName
            m_ServiceCertFindType = findType
            m_ServiceCertFindValue = findValue
            m_Anonymous = True
        End Sub

        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, True, Nothing)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal applicationName As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, True, applicationName)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal useProviders As Boolean, ByVal applicationName As String)
            ConfigureMessageSecurity(serviceCert, StoreLocation.LocalMachine, StoreName.My, useProviders, applicationName)
        End Sub
        Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal useProviders As Boolean, ByVal applicationName As String)
            ConfigureMessageSecurity(location, storeName, X509FindType.FindBySubjectName, serviceCert, useProviders, applicationName)
        End Sub
        <MethodImpl(MethodImplOptions.Synchronized)> _
        Private Sub ConfigureMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object, ByVal useProviders As Boolean, ByVal applicationName As String)
            m_ServiceCertLocation = location
            m_ServiceCertStoreName = storeName
            m_ServiceCertFindType = findType
            m_ServiceCertFindValue = findValue
            m_UseProviders = useProviders
            m_ApplicationName = applicationName
            m_Anonymous = False
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public MustOverride Sub SetBinding(ByVal binding As NetOnewayRelayBinding)

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public MustOverride Sub SetBinding(ByVal bindingConfigName As String)
        Protected MustOverride Function GetBinding() As NetOnewayRelayBinding


        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Subscribe()
            Dim serviceType As Type
            If m_SericeType IsNot Nothing Then
                serviceType = m_SericeType
            Else
                serviceType = m_SingletonInstance.GetType()
            End If
            Dim interfaces() As Type = serviceType.GetInterfaces()
            For Each interfaceType As Type In interfaces
                If interfaceType.GetCustomAttributes(GetType(ServiceContractAttribute), False).Length = 1 Then
                    Subscribe(interfaceType)
                End If
            Next interfaceType
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Subscribe(ByVal contractType As Type)
            Dim operations() As String = GetOperations(contractType)

            For Each operationName As String In operations
                Subscribe(contractType, operationName)
            Next operationName
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Overridable Sub Subscribe(ByVal contractType As Type, ByVal operation As String)
            Debug.Assert(String.IsNullOrEmpty(operation) = False)

            If Hosts.ContainsKey(contractType) = False Then
                Hosts(contractType) = New Dictionary(Of String, ServiceBusHost)()

                Dim operations() As String = GetOperations(contractType)
                For Each operationName As String In operations
                    Hosts(contractType)(operationName) = Nothing
                Next operationName
            End If
            If Hosts(contractType)(operation) Is Nothing Then
                If m_SericeType IsNot Nothing Then
                    Hosts(contractType)(operation) = New ServiceBusHost(m_SericeType)
                Else
                    Hosts(contractType)(operation) = New ServiceBusHost(m_SingletonInstance)
                End If

                AddHostEndpoints(contractType, operation)

                'Configure service bus credentials for the host
                If m_ServiceBusPassword IsNot Nothing Then
                    Hosts(contractType)(operation).SetServiceBusPassword(m_ServiceBusPassword)
                End If
                If m_ServiceBusCertFindValue IsNot Nothing Then
                    Hosts(contractType)(operation).SetServiceBusCertificate(m_ServiceBusCertFindValue, m_ServiceBusCertLocation, m_ServiceBusCertStoreName, m_ServiceBusCertFindType)
                End If

                'Configure message security
                If m_Anonymous Then
                    Hosts(contractType)(operation).ConfigureAnonymousMessageSecurity(m_ServiceCertLocation, m_ServiceCertStoreName, m_ServiceCertFindType, m_ServiceCertFindValue)
                Else
                    Hosts(contractType)(operation).ConfigureMessageSecurity(m_ServiceCertLocation, m_ServiceCertStoreName, m_ServiceCertFindType, m_ServiceCertFindValue, m_UseProviders, m_ApplicationName)
                End If

                Hosts(contractType)(operation).Open()
            End If
        End Sub

        Protected Overridable Sub AddHostEndpoints(ByVal contractType As Type, ByVal operation As String)
            Dim binding As Binding = GetBinding()

            For Each baseAddress As String In BaseAddresses
                Dim address As String = baseAddress.ToString & contractType.ToString & _
                    "/" & operation & "/"
                Hosts(contractType)(operation).AddServiceEndpoint(contractType, binding, address)
            Next baseAddress
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Unsubscribe()
            For Each contractType As Type In Hosts.Keys
                Unsubscribe(contractType)
            Next contractType
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Unsubscribe(ByVal contractType As Type)
            Dim operations() As String = GetOperations(contractType)

            For Each operationName As String In operations
                Unsubscribe(contractType, operationName)
            Next operationName
        End Sub

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Overridable Sub Unsubscribe(ByVal contractType As Type, ByVal operation As String)
            Debug.Assert(String.IsNullOrEmpty(operation) = False)

            If Hosts.ContainsKey(contractType) = False Then
                Return
            End If
            If Hosts(contractType)(operation) IsNot Nothing Then
                Hosts(contractType)(operation).Close()
                Hosts(contractType).Remove(operation)
                Hosts(contractType)(operation) = Nothing
            End If
        End Sub

        Friend Shared Function GetOperations(ByVal contract As Type) As String()
            Dim methods() As MethodInfo = contract.GetMethods(BindingFlags.Public Or BindingFlags.FlattenHierarchy Or BindingFlags.Instance)
            Dim operations As New List(Of String)(methods.Length)

            Dim add As Action(Of MethodInfo) = Function(method) AnonymousMethod1(method, operations)
            methods.ForEach(add)
            Return operations.ToArray()
        End Function

        Private Shared Function AnonymousMethod1(ByVal method As Object, ByVal operations As List(Of String)) As Boolean
            Debug.Assert((Not operations.Contains(method.Name)))
            operations.Add(method.Name)
            Return True
        End Function

        <MethodImpl(MethodImplOptions.Synchronized)> _
        Public Sub Abort()
            For Each contractType As Type In Hosts.Keys
                For Each host As ServiceHost In Hosts(contractType).Values
                    If host IsNot Nothing Then
                        host.Abort()
                    End If
                Next host
            Next contractType
        End Sub

        Public Property Credential() As TransportClientEndpointBehavior Implements IServiceBusProperties.Credential
            Get
                If m_Credential Is Nothing Then
                    m_Credential = New TransportClientEndpointBehavior()
                End If
                Return m_Credential
            End Get
            Set(ByVal value As TransportClientEndpointBehavior)
                m_Credential = value
            End Set
        End Property

        Public ReadOnly Property Addresses() As Uri() Implements IServiceBusProperties.Addresses
            Get
                'INSTANT VB NOTE: The local variable addresses was renamed since Visual Basic will not allow local variables with the same name as their enclosing function or property:
                Dim addresses_Renamed As New List(Of Uri)()

                For Each dictionary As Dictionary(Of String, ServiceBusHost) In Hosts.Values
                    For Each host As ServiceBusHost In dictionary.Values
                        For Each endpoint As ServiceEndpoint In host.Description.Endpoints
                            addresses_Renamed.Add(endpoint.Address.Uri)
                        Next endpoint
                    Next host
                Next dictionary
                Return addresses_Renamed.ToArray()
            End Get
        End Property
    End Class
End Namespace
