﻿' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net



Imports Microsoft.VisualBasic
Imports System
Imports System.ServiceModel
Imports Microsoft.ServiceBus
Imports System.Diagnostics
Imports System.ServiceModel.Description
Imports System.ServiceModel.Channels
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Security
Imports System.Reflection
Imports System.Collections.Generic
Imports System.Configuration
Imports System.Web.Configuration
Imports System.Web.Security


Namespace ServiceModelEx.ServiceBus
   Public Class ServiceBusHost
	   Inherits ServiceHost
	   Implements IServiceBusProperties
	  Public Sub New(ByVal singletonInstance As Object, ParamArray ByVal baseAddresses() As Uri)
		  MyBase.New(singletonInstance,baseAddresses)
	  End Sub
	  Public Sub New(ByVal serviceType As Type, ParamArray ByVal baseAddresses() As Uri)
		  MyBase.New(serviceType,baseAddresses)
	  End Sub
	  Protected Overrides Sub OnOpening()
		 If Credentials.ServiceCertificate.Certificate Is Nothing Then
			ConfigureAnonymousMessageSecurity()
		 End If
		 MyBase.OnOpening()
	  End Sub
	  Protected Sub ConfigureAnonymousMessageSecurity()
		 ConfigureAnonymousMessageSecurity("",StoreLocation.LocalMachine,StoreName.My)
	  End Sub
	  Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String)
		 ConfigureAnonymousMessageSecurity(serviceCert,StoreLocation.LocalMachine,StoreName.My)
	  End Sub
	  Public Sub ConfigureAnonymousMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName)
		 If serviceCert Is String.Empty Then
			serviceCert = ServiceBusHelper.ExtractSolution(Description.Endpoints(0).Address.Uri)
		 End If
		 ConfigureAnonymousMessageSecurity(location,storeName,X509FindType.FindBySubjectName,serviceCert)
	  End Sub
	  Public Sub ConfigureAnonymousMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object)
		 Credentials.ServiceCertificate.SetCertificate(location,storeName,findType,findValue)
		 Authorization.PrincipalPermissionMode = PrincipalPermissionMode.None

		 For Each endpoint As ServiceEndpoint In Description.Endpoints
			ServiceBusHelper.ConfigureBinding(endpoint.Binding)
		 Next endpoint
	  End Sub
	  Public Sub ConfigureMessageSecurity()
		 ConfigureMessageSecurity("",StoreLocation.LocalMachine,StoreName.My,True,Nothing)
	  End Sub
	  Public Sub ConfigureMessageSecurity(ByVal serviceCert As String)
		 ConfigureMessageSecurity(serviceCert,StoreLocation.LocalMachine,StoreName.My,True,Nothing)
	  End Sub
	  Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal applicationName As String)
		 ConfigureMessageSecurity(serviceCert,StoreLocation.LocalMachine,StoreName.My,True,applicationName)
	  End Sub
	  Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal useProviders As Boolean, ByVal applicationName As String)
		 ConfigureMessageSecurity(serviceCert,StoreLocation.LocalMachine,StoreName.My,useProviders,applicationName)
	  End Sub
	  Public Sub ConfigureMessageSecurity(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal useProviders As Boolean, ByVal applicationName As String)
		 If serviceCert Is String.Empty Then
			serviceCert = ServiceBusHelper.ExtractSolution(Description.Endpoints(0).Address.Uri)
		 End If
		 ConfigureMessageSecurity(location,storeName,X509FindType.FindBySubjectName,serviceCert,useProviders,applicationName)
	  End Sub

	  Private Shared Sub EnableRoleManagerInConfig()
		 Try
			'Get the role manager section.
			Dim roleManagerSection As RoleManagerSection = CType(ConfigurationManager.GetSection("system.web/roleManager"), RoleManagerSection)

			'Use reflection to enable changing it
			Dim fieldInfo As FieldInfo = GetType(System.Configuration.ConfigurationElement).GetField("_bReadOnly",BindingFlags.Instance Or BindingFlags.NonPublic)
			fieldInfo.SetValue(roleManagerSection,False)

			roleManagerSection.Enabled = True
		 Catch
		 End Try
	  End Sub
	  Public Sub ConfigureMessageSecurity(ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType, ByVal findValue As Object, ByVal useProviders As Boolean, ByVal applicationName As String)
		 Credentials.ServiceCertificate.SetCertificate(location,storeName,findType,findValue)

		 For Each endpoint As ServiceEndpoint In Description.Endpoints
			ServiceBusHelper.ConfigureBinding(endpoint.Binding,False)
		 Next endpoint
		 If useProviders Then
			Authorization.PrincipalPermissionMode = PrincipalPermissionMode.UseAspNetRoles
			Credentials.UserNameAuthentication.UserNamePasswordValidationMode = UserNamePasswordValidationMode.MembershipProvider

			EnableRoleManagerInConfig()

			Dim application As String

			If String.IsNullOrEmpty(applicationName) Then
			   applicationName = Membership.ApplicationName
			End If
			If String.IsNullOrEmpty(applicationName) OrElse applicationName = "/" Then
			   If String.IsNullOrEmpty(System.Reflection.Assembly.GetEntryAssembly().GetName().Name) Then
				  application = AppDomain.CurrentDomain.FriendlyName
			   Else
				  application = System.Reflection.Assembly.GetEntryAssembly().GetName().Name
			   End If
			Else
			   application = applicationName
			End If
			Membership.ApplicationName = application
			Roles.ApplicationName = application
		 End If
	  End Sub
	  Private Property Credential() As TransportClientEndpointBehavior Implements IServiceBusProperties.Credential
		 Get
			Dim credentials As TransportClientEndpointBehavior = Description.Endpoints(0).Behaviors.Find(Of TransportClientEndpointBehavior)()
			'Sanity check
			For Each endpoint As ServiceEndpoint In Description.Endpoints
			   Debug.Assert(endpoint.Behaviors.Find(Of TransportClientEndpointBehavior)() Is credentials)
			Next endpoint
			Return credentials
		 End Get
		 Set(ByVal value As TransportClientEndpointBehavior)
			For Each endpoint As ServiceEndpoint In Description.Endpoints
			   Debug.Assert(endpoint.Behaviors.Contains(GetType(TransportClientEndpointBehavior)) = False,"Do not add credentials mutiple times")
			   endpoint.Behaviors.Add(value)
			Next endpoint
		 End Set
	  End Property
	  Private ReadOnly Property Addresses() As Uri() Implements IServiceBusProperties.Addresses
		 Get
'INSTANT VB NOTE: The local variable addresses was renamed since Visual Basic will not allow local variables with the same name as their enclosing function or property:
			Dim addresses_Renamed As New List(Of Uri)()

			For Each endpoint As ServiceEndpoint In Description.Endpoints
			   addresses_Renamed.Add(endpoint.Address.Uri)
			Next endpoint
			Return addresses_Renamed.ToArray()
		 End Get
	  End Property
   End Class
End Namespace
