﻿' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.ServiceModel
Imports System.Diagnostics
Imports System.Collections.Generic
Imports Microsoft.ServiceBus
Imports System.Reflection
Imports System.ServiceModel.Channels
Imports System.Runtime.CompilerServices
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Description

Namespace ServiceModelEx.ServiceBus
   Public MustInherit Class EventRelayClientBase(Of T As Class)
	   Implements IServiceBusProperties, IDisposable
	  'State management 
	  Public Event Closed As EventHandler
	  Public Event Closing As EventHandler
	  Public Event Opened As EventHandler
	  Public Event Opening As EventHandler

	  Private privateState As CommunicationState
	  Public Property State() As CommunicationState
		  Get
			  Return privateState
		  End Get
		  Private Set(ByVal value As CommunicationState)
			  privateState = value
		  End Set
	  End Property

	  'Service bus items
	  Private ReadOnly BaseAddress As String

	  Private privateBinding As NetOnewayRelayBinding
	  Public Property Binding() As NetOnewayRelayBinding
		  Get
			  Return privateBinding
		  End Get
		  Protected Set(ByVal value As NetOnewayRelayBinding)
			  privateBinding = value
		  End Set
	  End Property

	  Private m_Proxies As Dictionary(Of String,T)

	  'For message security
	  Private ReadOnly ServiceUsername As String
	  Private ReadOnly ServicePassword As String
	  Private ReadOnly Anonymous As Boolean

	  'For serivce cert lookup
	  Private m_ServiceCertLocation As StoreLocation
	  Private m_ServiceCertStoreName As StoreName
	  Private m_ServiceCertFindType As X509FindType
	  Private m_ServiceCertFindValue As Object

	  'Service bus credentials
	  Private m_ServiceBusPassword As String
	  Private m_ServiceBusCertLocation As StoreLocation
	  Private m_ServiceBusCertStoreName As StoreName
	  Private m_ServiceBusCertFindType As X509FindType
	  Private m_ServiceBusCertFindValue As Object

	  Public Sub New(ByVal solutionBaseAddress As String, ByVal username As String, ByVal password As String)
		  Me.New(solutionBaseAddress,Nothing,username,password)
	  End Sub
	  Public Sub New(ByVal solutionBaseAddress As String, ByVal binding As NetOnewayRelayBinding, ByVal username As String, ByVal password As String)
		  Me.New(solutionBaseAddress,binding)
		 ServiceUsername = username
		 ServicePassword = password
		 Anonymous = False
	  End Sub

	  Public Sub New(ByVal solutionBaseAddress As String)
		  Me.New(solutionBaseAddress,New NetOnewayRelayBinding())
	  End Sub
	  Public Sub New(ByVal solutionBaseAddress As String, ByVal binding As NetOnewayRelayBinding)
		 State = CommunicationState.Faulted

		 Debug.Assert(solutionBaseAddress.Contains(ServiceBusEnvironment.DefaultRelayHostName))
		 If solutionBaseAddress.EndsWith("/") = False Then
			solutionBaseAddress &= "/"
		 End If
		 BaseAddress = solutionBaseAddress

		 Binding = binding

		 Anonymous = True

		 SetServiceCertificate("")

		 State = CommunicationState.Created
	  End Sub

	  <MethodImpl(MethodImplOptions.Synchronized)> _
	  Public Sub Open()
		 If State <> CommunicationState.Created Then
			Return
		 End If
		 Try
			RaiseEvent Opening(Me,EventArgs.Empty)
			ServiceBusHelper.ConfigureBinding(Binding,Anonymous)

			m_Proxies = New Dictionary(Of String,T)()

			Dim properties As IServiceBusProperties = Me

			For Each uri As Uri In properties.Addresses
			   Dim identity As EndpointIdentity = New DnsEndpointIdentity(m_ServiceCertFindValue.ToString())
			   Dim address As New EndpointAddress(uri,identity)
			   Dim factory As New ChannelFactory(Of T)(Binding,address)

			   'Set credentials for message security (if needed)
			   factory.Credentials.UserName.UserName = ServiceUsername 'could be null
			   factory.Credentials.UserName.Password = ServicePassword 'could be null

			   'Set service cert to secure message 
			   Dim behavior As ClientCredentials = factory.Endpoint.Behaviors.Find(Of ClientCredentials)()
			   behavior.ServiceCertificate.SetDefaultCertificate(m_ServiceCertLocation,m_ServiceCertStoreName,m_ServiceCertFindType,m_ServiceCertFindValue)

			   'Set service bus creds
			   If properties.Credential Is Nothing Then
				  If m_ServiceBusPassword IsNot Nothing Then
					 factory.SetServiceBusPassword(m_ServiceBusPassword)
				  End If
				  If m_ServiceBusCertFindValue IsNot Nothing Then
					 factory.SetServiceBusCertificate(m_ServiceBusCertFindValue,m_ServiceBusCertLocation,m_ServiceBusCertStoreName,m_ServiceBusCertFindType)
				  End If
			   Else
				  Debug.Assert(m_ServiceBusPassword Is Nothing)
				  Debug.Assert(m_ServiceBusCertFindValue Is Nothing)
				  factory.Endpoint.Behaviors.Add(properties.Credential)
			   End If
			   Dim methodName As String = uri.Segments(uri.Segments.Length-1)
			   methodName = methodName.Replace("/","")
			   m_Proxies(methodName) = factory.CreateChannel()
			   Dim proxy As ICommunicationObject = TryCast(m_Proxies(methodName), ICommunicationObject)
			   proxy.Open()
			Next uri
                State = CommunicationState.Opened

			RaiseEvent Opened(Me,EventArgs.Empty)
		 Catch
			State = CommunicationState.Faulted
		 End Try
	  End Sub
	  Public Sub Close()
            If State <> CommunicationState.Opened Then
                Return
            End If
		 Try
			RaiseEvent Closing(Me,EventArgs.Empty)

			For Each proxy As ICommunicationObject In m_Proxies.Values
			   proxy.Close()
			Next proxy
                State = CommunicationState.Closed
			RaiseEvent Closed(Me,EventArgs.Empty)
		 Catch
			State = CommunicationState.Faulted
		 End Try
	  End Sub
	  Public Sub Abort()
		 Try
			For Each proxy As ICommunicationObject In m_Proxies.Values
			   proxy.Abort()
			Next proxy
		 Finally
			State = CommunicationState.Faulted
		 End Try
	  End Sub

	  Public ReadOnly Property Channel() As T
		 Get
                If State <> CommunicationState.Opened Then
                    Open()
                End If
			Dim frame As New StackFrame(1)
			Return m_Proxies(frame.GetMethod().Name)
		 End Get
	  End Property

	  Public Sub SetServiceCertificate(ByVal serviceCert As String)
		 SetServiceCertificate(serviceCert,StoreLocation.LocalMachine,StoreName.My)
	  End Sub
	  Public Sub SetServiceCertificate(ByVal serviceCert As String, ByVal location As StoreLocation, ByVal storeName As StoreName)
		 If serviceCert Is String.Empty Then
			serviceCert = ServiceBusHelper.ExtractSolution(New Uri(BaseAddress))
		 End If
		 SetServiceCertificate(serviceCert,location,storeName,X509FindType.FindBySubjectName)
	  End Sub
	  Public Sub SetServiceCertificate(ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
		 m_ServiceCertLocation = location
		 m_ServiceCertStoreName = storeName
		 m_ServiceCertFindType = findType
		 m_ServiceCertFindValue = findValue
	  End Sub


	  Public Sub SetServiceBusCertificate(ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
		 m_ServiceBusCertFindValue = findValue
		 m_ServiceBusCertLocation = location
		 m_ServiceBusCertStoreName = storeName
		 m_ServiceBusCertFindType = findType
	  End Sub
	  Public Sub SetServiceBusCertificate(ByVal subjectName As String)
		 SetServiceBusCertificate(subjectName,StoreLocation.LocalMachine,StoreName.My,X509FindType.FindBySubjectName)
	  End Sub

	  Public Sub SetServiceBusPassword(ByVal password As String)
		 m_ServiceBusPassword = password
	  End Sub

	  Private Sub Dispose() Implements IDisposable.Dispose
		 Close()
	  End Sub

	  Private privateCredential As TransportClientEndpointBehavior
	  Private Property Credential() As TransportClientEndpointBehavior Implements IServiceBusProperties.Credential
		  Get
			  Return privateCredential
		  End Get
		  Set(ByVal value As TransportClientEndpointBehavior)
			  privateCredential = value
		  End Set
	  End Property

	  Private ReadOnly Property Addresses() As Uri() Implements IServiceBusProperties.Addresses
		 Get
                Dim addressList As New List(Of Uri)()

			Dim methods() As MethodInfo = GetType(T).GetMethods(BindingFlags.Public Or BindingFlags.FlattenHierarchy Or BindingFlags.Instance)

			For Each method As MethodInfo In methods
                    addressList.Add(New Uri(BaseAddress & GetType(T).ToString & "/" & method.Name & "/"))
			Next method
			Return addressList.ToArray()
		 End Get
	  End Property
   End Class
End Namespace