﻿' 2009 IDesign Inc.
'Questions? Comments? go to 
'http://www.idesign.net


Imports Microsoft.VisualBasic
Imports System
Imports System.ServiceModel
Imports System.ServiceModel.Description
Imports Microsoft.ServiceBus
Imports System.Collections.ObjectModel
Imports System.Diagnostics
Imports ServiceModelEx
Imports System.ServiceModel.Configuration
Imports System.Configuration
Imports System.Security.Cryptography.X509Certificates
Imports System.ServiceModel.Channels

Namespace ServiceModelEx.ServiceBus
    Public Module ServiceBusHelper
        Private Sub SetServiceBusCertificate(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal subjectName As String)
            If String.IsNullOrEmpty(subjectName) Then
                If TypeOf endpoints(0).Binding Is NetTcpRelayBinding OrElse TypeOf endpoints(0).Binding Is WSHttpRelayBinding OrElse TypeOf endpoints(0).Binding Is NetOnewayRelayBinding Then
                    subjectName = ExtractSolution(endpoints(0).Address.Uri)
                End If
            End If
            'Sanity check: 
            For Each endpoint As ServiceEndpoint In endpoints
                If TypeOf endpoint.Binding Is NetTcpRelayBinding OrElse TypeOf endpoint.Binding Is WSHttpRelayBinding OrElse TypeOf endpoint.Binding Is NetOnewayRelayBinding Then
                    Debug.Assert(subjectName = ExtractSolution(endpoint.Address.Uri))
                End If
            Next endpoint

            SetServiceBusCertificate(endpoints, subjectName, StoreLocation.LocalMachine, StoreName.My, X509FindType.FindBySubjectName)
        End Sub
        Private Sub SetServiceBusCertificate(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            Dim behavior As New TransportClientEndpointBehavior()
            behavior.CredentialType = TransportClientCredentialType.X509Certificate
            behavior.Credentials.ClientCertificate.SetCertificate(location, storeName, findType, findValue)

            SetBehavior(endpoints, behavior)
        End Sub

        Private Sub SetServiceBusPassword(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal password As String)
            Dim solution As String = Nothing

            If TypeOf endpoints(0).Binding Is NetTcpRelayBinding OrElse TypeOf endpoints(0).Binding Is WSHttpRelayBinding OrElse TypeOf endpoints(0).Binding Is NetOnewayRelayBinding Then
                solution = ExtractSolution(endpoints(0).Address.Uri)
            End If

            'Sanity check: 
            For Each endpoint As ServiceEndpoint In endpoints
                If TypeOf endpoint.Binding Is NetTcpRelayBinding OrElse TypeOf endpoint.Binding Is WSHttpRelayBinding OrElse TypeOf endpoint.Binding Is NetOnewayRelayBinding Then
                    Debug.Assert(solution = ExtractSolution(endpoint.Address.Uri))
                End If
            Next endpoint
            SetServiceBusPassword(endpoints, solution, password)
        End Sub
        Private Sub SetServiceBusPassword(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal solution As String, ByVal password As String)
            Dim behavior As New TransportClientEndpointBehavior()
            behavior.CredentialType = TransportClientCredentialType.UserNamePassword
            behavior.Credentials.UserName.UserName = solution
            behavior.Credentials.UserName.Password = password

            SetBehavior(endpoints, behavior)
        End Sub

        Private Sub SetBehavior(ByVal endpoints As Collection(Of ServiceEndpoint), ByVal credential As TransportClientEndpointBehavior)
            For Each endpoint As ServiceEndpoint In endpoints
                If TypeOf endpoint.Binding Is NetTcpRelayBinding OrElse TypeOf endpoint.Binding Is WSHttpRelayBinding OrElse TypeOf endpoint.Binding Is NetOnewayRelayBinding Then
                    endpoint.Behaviors.Add(credential)
                End If
            Next endpoint
        End Sub
        Friend Sub ConfigureBinding(ByVal binding As Binding)
            ConfigureBinding(binding, True)
        End Sub
        Friend Sub ConfigureBinding(ByVal binding As Binding, ByVal anonymous As Boolean)
            If TypeOf binding Is NetTcpRelayBinding Then
                Dim tcpBinding As NetTcpRelayBinding = CType(binding, NetTcpRelayBinding)
                tcpBinding.Security.Mode = EndToEndSecurityMode.Message
                If anonymous Then
                    tcpBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
                Else
                    tcpBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
                End If

                tcpBinding.ConnectionMode = TcpRelayConnectionMode.Hybrid
                tcpBinding.ReliableSession.Enabled = True

                Return
            End If
            If TypeOf binding Is WSHttpRelayBinding Then
                Dim wsBinding As WSHttpRelayBinding = CType(binding, WSHttpRelayBinding)
                wsBinding.Security.Mode = EndToEndSecurityMode.Message
                If anonymous Then
                    wsBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
                Else
                    wsBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
                End If
                wsBinding.ReliableSession.Enabled = True

                Return
            End If
            If TypeOf binding Is NetOnewayRelayBinding Then
                Dim onewayBinding As NetOnewayRelayBinding = CType(binding, NetOnewayRelayBinding)
                onewayBinding.Security.Mode = EndToEndSecurityMode.Message
                If anonymous Then
                    onewayBinding.Security.Message.ClientCredentialType = MessageCredentialType.None
                Else
                    onewayBinding.Security.Message.ClientCredentialType = MessageCredentialType.UserName
                End If
                Return
            End If
            Throw New InvalidOperationException(binding.GetType().ToString & " is unsupported")
        End Sub

        Public Function ExtractSolution(ByVal address As Uri) As String
            Return address.Host.Split("."c)(0)
        End Function

        Public Function ExtractSolutionFromConfig(ByVal endpointName As String) As String
            Dim config As Configuration = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None)
            Dim sectionGroup As ServiceModelSectionGroup = ServiceModelSectionGroup.GetSectionGroup(config)

            For Each endpointElement As ChannelEndpointElement In sectionGroup.Client.Endpoints
                If endpointElement.Name = endpointName Then
                    Return ExtractSolution(endpointElement.Address)
                End If
            Next endpointElement
            Return Nothing
        End Function
        Public Function ExtractSolutionFromConfig(ByVal serviceType As Type) As String
            Dim config As Configuration = ConfigurationManager.OpenExeConfiguration(ConfigurationUserLevel.None)
            Dim sectionGroup As ServiceModelSectionGroup = ServiceModelSectionGroup.GetSectionGroup(config)

            For Each serviceElement As ServiceElement In sectionGroup.Services.Services
                If serviceElement.Name = serviceType.ToString() Then
                    Return ExtractSolution(serviceElement.Endpoints(0).Address)
                End If
            Next serviceElement
            Return Nothing
        End Function

        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal proxy As ClientBase(Of T), ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            If proxy.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Proxy is already opened")
            End If
            proxy.ChannelFactory.SetServiceBusCertificate(findValue, location, storeName, findType)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal proxy As ClientBase(Of T), ByVal subjectName As String)
            If proxy.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Proxy is already opened")
            End If
            proxy.ChannelFactory.SetServiceBusCertificate(subjectName)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal proxy As ClientBase(Of T))
            proxy.ChannelFactory.SetServiceBusCertificate()
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(Of T As Class)(ByVal proxy As ClientBase(Of T), ByVal solution As String, ByVal password As String)
            If proxy.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Proxy is already opened")
            End If
            proxy.ChannelFactory.SetServiceBusPassword(solution, password)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(Of T As Class)(ByVal proxy As ClientBase(Of T), ByVal password As String)
            If proxy.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Proxy is already opened")
            End If
            proxy.ChannelFactory.SetServiceBusPassword(password)
        End Sub

        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal factory As ChannelFactory(Of T), ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            If factory.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Factory is already opened")
            End If
            Dim endpoints As New Collection(Of ServiceEndpoint)()
            endpoints.Add(factory.Endpoint)
            SetServiceBusCertificate(factory, findValue, location, storeName, findType)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal factory As ChannelFactory(Of T), ByVal subjectName As String)
            If factory.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Factory is already opened")
            End If
            Dim endpoints As New Collection(Of ServiceEndpoint)()
            endpoints.Add(factory.Endpoint)
            SetServiceBusCertificate(endpoints, subjectName)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(Of T As Class)(ByVal factory As ChannelFactory(Of T))
            SetServiceBusCertificate(factory, Nothing)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(Of T As Class)(ByVal factory As ChannelFactory(Of T), ByVal password As String)
            If factory.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Factory is already opened")
            End If
            Dim endpoints As New Collection(Of ServiceEndpoint)()
            endpoints.Add(factory.Endpoint)
            SetServiceBusPassword(endpoints, password)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(Of T As Class)(ByVal factory As ChannelFactory(Of T), ByVal solution As String, ByVal password As String)
            If factory.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Factory is already opened")
            End If
            Dim endpoints As New Collection(Of ServiceEndpoint)()
            endpoints.Add(factory.Endpoint)
            SetServiceBusPassword(endpoints, solution, password)
        End Sub

        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(ByVal host As ServiceHost, ByVal findValue As Object, ByVal location As StoreLocation, ByVal storeName As StoreName, ByVal findType As X509FindType)
            If host.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Host is already opened")
            End If
            SetServiceBusCertificate(host.Description.Endpoints, findValue, location, storeName, findType)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(ByVal host As ServiceHost, ByVal subjectName As String)
            If host.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Host is already opened")
            End If
            SetServiceBusCertificate(host.Description.Endpoints, subjectName)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusCertificate(ByVal host As ServiceHost)
            SetServiceBusCertificate(host.Description.Endpoints, Nothing)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(ByVal host As ServiceHost, ByVal solution As String, ByVal password As String)
            If host.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Host is already opened")
            End If
            SetServiceBusPassword(host.Description.Endpoints, solution, password)
        End Sub
        <System.Runtime.CompilerServices.Extension()> _
        Public Sub SetServiceBusPassword(ByVal host As ServiceHost, ByVal password As String)
            If host.State = CommunicationState.Opened Then
                Throw New InvalidOperationException("Host is already opened")
            End If
            SetServiceBusPassword(host.Description.Endpoints, password)
        End Sub
    End Module
End Namespace





